/** @file fifo_media.cpp
 * Mediator for united FIFO model.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#include <multififo_media.h>

#include <sync.h>

#include <interface.h>
#include <vpi_media.h>

#include <iostream>

namespace cpptesk {
namespace examples {
namespace fifo {

using namespace cpptesk::hw;

MultififoMediator::MultififoMediator()
{
    debug(DEBUG_ERROR);
    // Registration of correspondence between serializers, deserializers and ifaces.
    CPPTESK_SET_INPUT_ADAPTER(iface1a, MultififoMediator::serialize_iface1a);
    CPPTESK_SET_INPUT_ADAPTER(iface1b, MultififoMediator::serialize_iface1b);
    CPPTESK_SET_INPUT_ADAPTER(iface2, MultififoMediator::serialize_iface2);
    CPPTESK_SET_OUTPUT_ADAPTER(iface3, MultififoMediator::deserialize_iface3);

    CPPTESK_SET_INNER_IFACE_ADAPTER(FIFOMediator, fifo0, fifo0.iface3, MultififoMediator::deserialize_inner_iface0);
    CPPTESK_SET_INNER_IFACE_ADAPTER(FIFOMediator, fifo1, fifo1.iface3, MultififoMediator::deserialize_inner_iface1);

    // Set maximum timeout.
    setReactionTimeout(MultififoMediator::REACTION_TIMEOUT);

    // Initialization of structures aimed to keep DUT' values on the wires.
    clear_inputs(&inputs);
    clear_outputs(&outputs);

    // Registration of arbiter of output iface3.
    CPPTESK_SET_ARBITER(iface3, matcher_iface3);

    // Detectors of extra reactions should be started on the first cycle.
    CALL_OUTPUT_LISTENER(MultififoMediator::listen_iface3, iface3);
}

MultififoMediator::~MultififoMediator() {}

bool MultififoMediator::isIface1aReady() const
{
    return Multififo::isIface1aReady();
}
bool MultififoMediator::isIface1bReady() const
{
    return Multififo::isIface1bReady();
}
bool MultififoMediator::isIface2Ready() const
{
    return Multififo::isIface2Ready();
}

CPPTESK_DEFINE_STIMULUS(MultififoMediator::serialize_iface1a)
{
    // Copy data to the local variable.
    InputData msg = CPPTESK_CAST_MESSAGE(InputData);

    // Start of serialization processing.
    START();

#ifdef DEBUG
    std::cout << "MultififoMediator::serialize_iface1a" << std::endl;
#endif

    // Capture iface1a.
    iface.capture();

    // Set data to the wires of DUT.
    inputs.CE_WR0 = 1;
    inputs.DI0 = msg.get_data();

    // Release of iface1a.
    iface.release();

    // Wait one cycle of simulation.
    CYCLE();

    // Finish of serialization processing.
    STOP();
}

CPPTESK_DEFINE_STIMULUS(MultififoMediator::serialize_iface1b)
{
    // Copy data to the local variable.
    InputData msg = CPPTESK_CAST_MESSAGE(InputData);

    // Start of serialization processing.
    START();

#ifdef DEBUG
    std::cout << "MultififoMediator::serialize_iface1b" << std::endl;
#endif

    // Capture iface1b.
    iface.capture();

    // Set data to the wires of DUT.
    inputs.CE_WR1 = 1;
    inputs.DI1 = msg.get_data();

    // Release of iface1b.
    iface.release();

    // Wait one cycle of simulation.
    CYCLE();

    // Finish of serialization processing.
    STOP();
}

CPPTESK_DEFINE_STIMULUS(MultififoMediator::serialize_iface2)
{
    // Copy data to the local variable.
    InputData msg = CPPTESK_CAST_MESSAGE(InputData);

    // Start of serialization processing.
    START();

#ifdef DEBUG
    std::cout << "MultififoMediator::serialize_iface2" << std::endl;
#endif

    // Capture iface2.
    iface.capture();

    // Set data to the wires of DUT.
    inputs.CE_RD = 1;

    // Release of iface2.
    iface.release();

    // Wait one cycle of simulation.
    CYCLE();

    // Finish of serialization processing.
    STOP();
}

CPPTESK_DEFINE_REACTION(MultififoMediator::deserialize_iface3)
{
    // Copy data to the local variable.
    OutputData &msg = CPPTESK_CAST_MESSAGE(OutputData);

    // Start of deserialization processing.
    START();

    // Wainting for reaction by means of DUT's DO_VAL wire.
    WAIT_REACTION(outputs.DO_VAL);

    // Output data are saved to the following check.
    msg.set_data(outputs.DO);

    // Jump to the next registered reaction (if it exists).
    NEXT_REACTION();

    // Finish of deserialization processing.
    STOP();
}

void MultififoMediator::initialize()
{
    ::start_testbench();
}

void MultififoMediator::finalize()
{
    ::stop_testbench();
}

void MultififoMediator::setInputs()
{
    ::map_state_down(&inputs);
}

void MultififoMediator::getOutputs()
{
    ::map_state_up(&outputs);
}

CPPTESK_DEFINE_PROCESS(MultififoMediator::deserialize_inner_iface0)
{
    OutputData msg = CPPTESK_CAST_MESSAGE(OutputData);

    START();

#ifdef DEBUG
    std::cout << "MultififoMediator::deserialize_inner_iface0" << std::endl;
#endif
printf("iface0: %x\n", msg.get_data());
    iface.capture();

    if(!fifo2.is_full()) {
        InputData data = InputData();
        data.set_data(msg.get_data());
        fifo2.start(&FIFO::push_msg, fifo2.iface1, data, FIFO::START_REQUEST_IFACE);
    } else { printf("but the fifo2 is full\n"); }

    iface.release();

    CYCLE_EX(fifo0);

    STOP();
}

CPPTESK_DEFINE_PROCESS(MultififoMediator::deserialize_inner_iface1)
{
    OutputData msg = CPPTESK_CAST_MESSAGE(OutputData);

    START();

#ifdef DEBUG
    std::cout << "MultififoMediator::deserialize_inner_iface1" << std::endl;
#endif
printf("iface1: %x\n", msg.get_data());
    iface.capture();

    if(!fifo2.is_full()) {
        while(!fifo2.isIface1Ready()) CYCLE();
        InputData data = InputData();
        data.set_data(msg.get_data());
        fifo2.start(&FIFO::push_msg, fifo2.iface1, data, FIFO::START_REQUEST_IFACE);
    } else { printf("but the fifo2 is full\n"); }

    iface.release();

    CYCLE_EX(fifo1);

    STOP();
}

void MultififoMediator::cycle()
{
    fifo0.cycle();
    fifo1.cycle();
    fifo2.cycle();
    Multififo::cycle();
}

void MultififoMediator::simulate()
{
    ::wait_for_check();
}

}}}
