/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Generalized model of a hardware component
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/assertion.hpp>
#include <hw/debug.hpp>

#include <assert.h>
#include <string>

#include <hw/process.hpp>
#include <hw/time.hpp>

/** @def CPPTESK_CAPTURE_COMPONENT
 * Captures a component.
 */
#define CPPTESK_CAPTURE_COMPONENT(component) \
   do { \
        CPPTESK_DEBUG_PRINT(::cpptesk::hw::DEBUG_INFO, "[CAPTURE] (component=" << component.getName() << ")"); \
        (component).capture(); \
    } while(false)

/** @def CPPTESK_RELEASE_COMPONENT
 * Releases a component.
 */
#define CPPTESK_RELEASE_COMPONENT(component) \
    do { \
        CPPTESK_DEBUG_PRINT(::cpptesk::hw::DEBUG_INFO, "[RELEASE] (component=" << component.getName() << ")"); \
        (component).release(); \
    } while(false)

#ifdef HW_SHORT_NAMES
    #define CAPTURE_COMPONENT(component) CPPTESK_CAPTURE_COMPONENT(component)
    #define RELEASE_COMPONENT(component) CPPTESK_RELEASE_COMPONENT(component)
#endif // HW_SHORT_NAMES

#define __CPPTESK_DEFAULT_COMPONENT_NAME "<unregistered>"

namespace cpptesk {
namespace hw {

class Component 
{
public:
    typedef enum { IDLE, LOCK, BUSY } status_t;

    Component(const std::string &name = __CPPTESK_DEFAULT_COMPONENT_NAME):
        status(IDLE), name(name), timer(0) {}

    virtual ~Component() {}

    const std::string& getName() const { return name; }
    void setName(const std::string &name) { this->name = name; }
    void updateName(const std::string &name) { if(this->name == __CPPTESK_DEFAULT_COMPONENT_NAME) { this->name = name; } }

    virtual void request(const Process &process)
    {
        CPPTESK_ASSERTION(status == IDLE, "Component " << name << " has been already requested");
        status = LOCK; owner = (Process*)&process;
    }

    virtual void capture()
    {
        CPPTESK_ASSERTION((status == LOCK || status == IDLE), "Component " << name << " has been been already captured");
        status = BUSY;
    }

    virtual void release()
    {
        CPPTESK_ASSERTION((status == BUSY || status == LOCK), "Component " << name << " has been already released");
        status = IDLE; timer = 0; owner = 0;
    }

    virtual status_t getStatus() const { return status; }

    virtual bool isReady() const { return status == IDLE; }

    virtual Process* getOwner() const { return owner; }

    virtual void cycle() { if(status == BUSY) { timer++; } }

    cycle_t getTimer() const { return timer; }

protected:

    Process *owner;

    status_t status;

    std::string name;

    cycle_t timer;
};

}} // namespace cpptesk::hw

