/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Model of a hardware interface
///////////////////////////////////////////////////////////////////////////////

#include <hw/assertion.hpp>
#include <hw/debug.hpp>
#include <hw/interface.hpp>
#include <hw/message.hpp>
#include <hw/process.hpp>

#include <algorithm>
#include <ostream>
#include <vector>

#define __CPPTESK_CHECK_ARBITER() \
   do { \
        if(!matcher) { \
            CPPTESK_DEBUG_PRINT(::cpptesk::hw::DEBUG_WARN, "Reaction arbiter is not set for the interface " << *this); \
        } \
   } while(false)

namespace cpptesk {
namespace hw {

void Interface::request(const Process &process, const Message &message)
{
    CPPTESK_DEBUG_PRINT(DEBUG_MORE, "[IFACE:REQUEST](iface=" << *this << ")");

    Component::request(process);
    this->message = (Message*)&message;
}

void Interface::capture()
{
    CPPTESK_DEBUG_PRINT(DEBUG_MORE, "[IFACE:CAPTURE](iface=" << *this << ")");

    Component::capture();
 }

void Interface::release()
{
    CPPTESK_DEBUG_PRINT(DEBUG_MORE, "[IFACE:RELEASE](iface=" << *this << ")");

    Component::release();
    message = 0;
}

void Interface::appendRequest(const Process &process, const Message &message)
{
    __CPPTESK_CHECK_ARBITER();

    if(matcher) { matcher->appendRequest(process, message); }
    CPPTESK_DEBUG_PRINT(DEBUG_MORE, "[ARBITER:APPEND](iface=" << *this << "): " << process);

    requestCount++;
}

bool Interface::chooseRequest(const Process &process, const Message &message) const
{
    __CPPTESK_CHECK_ARBITER();

    bool choose = matcher ? matcher->chooseRequest(process, message) : true;
    CPPTESK_DEBUG_PRINT(DEBUG_MORE, "[ARBITER:CHOOSE](iface=" << *this << "): " << process << ": " << (choose ? "YES" : "NO"));

    return choose;
}

void Interface::removeRequest(const Process &process, const Message &message)
{
    __CPPTESK_CHECK_ARBITER();

    if(matcher) { matcher->removeRequest(process, message); }
    CPPTESK_DEBUG_PRINT(DEBUG_MORE, "[ARBITER:REMOVE](iface=" << *this << "): " << process);

    requestCount--;
    taken = false;
}

bool Interface::takeReaction()
{
    CPPTESK_ASSERTION(requestCount > 0, "Reaction could not be taken, because there are no requests to take it");

    if(taken) { return false; }
    return taken = true;
}

}} // namespace cpptesk::hw
