/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Model of a hardware interface
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/assertion.hpp>
#include <hw/component.hpp>
#include <hw/matcher.hpp>
#include <hw/message.hpp>
#include <hw/process.hpp>

#include <algorithm>
#include <ostream>
#include <string>
#include <vector>

namespace cpptesk {
namespace hw {

class Interface: public Component
{
public:
    typedef enum { INPUT = 0x01, OUTPUT = 0x02, INOUT = INPUT | OUTPUT } inout_t;

    Interface(inout_t type = INOUT, const std::string &name = __CPPTESK_DEFAULT_COMPONENT_NAME):
        Component(name), message(0), type(type), matcher(0), requestCount(0), taken(false) {}

    virtual ~Interface() {}

    void registerReactionMatcher(ReactionMatcher &matcher)
    {
        this->matcher = &matcher;
    }

    inline bool isInput()  const { return (type & INPUT)  != 0; }
    inline bool isOutput() const { return (type & OUTPUT) != 0; }

    // Basically for input interfaces
    virtual void request(const Process &process, const Message &message);
    virtual void capture();
    virtual void release();

    virtual Message* getMessage() const { return message; }

    // Basically for output interfaces
    virtual void appendRequest(const Process &process, const Message &message);
    virtual bool chooseRequest(const Process &process, const Message &message) const;
    virtual void removeRequest(const Process &process, const Message &message);

    virtual int requestNumber() const
    {
        return requestCount;
    }

    /// Returns false if a reaction has been taken by someone else.
    virtual bool takeReaction();

    virtual bool isReactionTaken() const
    {
        return taken;
    }

protected:
    /// Current input message.
    Message *message;

    /// Type of the interface.
    inout_t type;

    /// Reaction matcher.
    ReactionMatcher *matcher;

    /// Number of requests to take a reaction.
    int requestCount;

    /// Flag that shows if a reaction is taken or not.
    bool taken;
};

class InputInterface: public Interface
{
public:
    InputInterface(const std::string &name = __CPPTESK_DEFAULT_COMPONENT_NAME): Interface(INPUT, name) {}
    virtual ~InputInterface() {}
};

class OutputInterface: public Interface
{
public:
    OutputInterface(const std::string &name = __CPPTESK_DEFAULT_COMPONENT_NAME): Interface(OUTPUT, name) {}
    virtual ~OutputInterface() {}
};

class InoutInterface: public Interface
{
public:
    InoutInterface(const std::string &name = __CPPTESK_DEFAULT_COMPONENT_NAME): Interface(INOUT, name) {}
    virtual ~InoutInterface() {}
};

inline std::ostream& operator << (std::ostream &out, const Interface &iface)
{
	return out << iface.getName();
}

}} // namespace cpptesk::hw

